/* /////////////////////////////////////////////////////////////////////////////
 * File:        custom_event_handler_test.cpp
 *
 * Purpose:     Implementation file for the custom_event_handler_test project.
 *
 * Created:     4th October 2004
 * Updated:     27th June 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* STLSoft Header Files */
#include <stlsoft.h>

/* ACESTL Header Files */
#include <acestl/acestl.hpp>
#include <acestl/custom_event_handler.hpp>
#include <acestl/time_string_access.hpp>

using acestl::custom_event_handler::event_id;

#if 0
# include <ACE/Map.h>
#endif /* 0 */

/* Standard C Header Files */

/* Standard C++ Header Files */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

#if 0
#if 0
void ace_new_handler()
#else /* ? 0 */
# include <new.h>
int ace_new_handler(size_t)
#endif /* 0 */
{
	throw std::bad_alloc("Out of memory");

#if 0
#else /* ? 0 */
	return 0;
#endif /* 0 */
}
#endif /* 0 */

/* ////////////////////////////////////////////////////////////////////////// */

class MyNotification
    : public acestl::custom_event_handler
{
public:
    typedef acestl::custom_event_handler    parent_class_type;
    typedef MyNotification                  class_type;

public:
    MyNotification(ACE_Reactor *reactor)
        : parent_class_type(reactor)
		, m_start(ACE_OS::gettimeofday())
    {}
    MyNotification()
    {}

public:
	void handle_cancel(long code, event_id id, void *arg)
	{
		fprintf(stdout, "Cancelled event by method: %ld, %p, %lu\n", code, id, reinterpret_cast<unsigned long>(arg));
	}

private:
    virtual int handle_timeout( const ACE_Time_Value    &current_time
                            ,   const void              *arg)
    {
        return 0;
    }

    virtual int handle_custom_event(const ACE_Time_Value    &current_time
                                ,   long                    event_code
                                ,   void                    *arg)
    {
		const ACE_Time_Value	delta = current_time - m_start;

        std::cout << "handle custom event: code: " << event_code 
				  << "; arg: " << reinterpret_cast<unsigned long>(arg)
				  << "; t: " << (delta.sec() * 1000 + delta.usec() / 1000)
				  << std::endl;

        return 0;
    }

private:
	const ACE_Time_Value	m_start;
};

/* ////////////////////////////////////////////////////////////////////////// */

class CancelClient
{
public:
	void report(long code, event_id id, void *arg)
	{
		std::cout << "Cancelled: "
				  << "code: " << code
				  << "; id: " << reinterpret_cast<unsigned long>(id)
				  << "; arg: " << reinterpret_cast<unsigned long>(arg)
				  << std::endl;
	}

	void proc(void *param, long code, event_id id, void *arg)
	{
		static_cast<CancelClient*>(param)->report(code, id, arg);
	}
};

/* ////////////////////////////////////////////////////////////////////////// */

static void cancel_proc(void *param, long code, event_id id, void *arg);

/* ////////////////////////////////////////////////////////////////////////// */

int main_(int /* argc */, char ** /*argv*/)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
#endif /* debug */

	/* . */
    event_id        id1, id2, id3, id4, id5, id6;
    MyNotification  *mn = new MyNotification(ACE_Reactor::instance());
    acestl_assert(0 == mn->has_custom_event(100));
    acestl_assert(0 == mn->has_custom_event(101));
    acestl_assert(0 == mn->has_custom_event(102));

    id1 = mn->schedule_custom_event(100, (void*)1000);
    id2 = mn->schedule_custom_event(100, ACE_Time_Value(1), (void*)1001);
    acestl_assert(0 != mn->has_custom_event(100));
    acestl_assert(0 == mn->has_custom_event(101));
    acestl_assert(0 == mn->has_custom_event(102));

    id3 = mn->schedule_custom_event(101, (void*)1002);
    acestl_assert(0 != mn->has_custom_event(100));
    acestl_assert(0 != mn->has_custom_event(101));
    acestl_assert(0 == mn->has_custom_event(102));

    id4 = mn->schedule_custom_event(102, (void*)1003);
    id5 = mn->schedule_custom_event(102, ACE_Time_Value(2), (void*)1004);
    id6 = mn->schedule_custom_event(102, ACE_Time_Value(2, 100000), (void*)1005);
    acestl_assert(0 != mn->has_custom_event(100));
    acestl_assert(0 != mn->has_custom_event(101));
    acestl_assert(0 != mn->has_custom_event(102));

#if defined(ACESTL_CUSTOM_EVENT_HANDLER_CANCEL_EVENTS_MEMBER_CLEANUP_SUPPORTx)
    mn->cancel_custom_events(102, mn, &MyNotification::handle_cancel);
#endif /* ACESTL_CUSTOM_EVENT_HANDLER_CANCEL_EVENTS_MEMBER_CLEANUP_SUPPORT */

    mn->cancel_custom_event(id5);
    mn->cancel_custom_events(101, cancel_proc, NULL);
    acestl_assert(0 != mn->has_custom_event(100));
    acestl_assert(0 == mn->has_custom_event(101));
    acestl_assert(0 != mn->has_custom_event(102));

    ACE_Reactor::instance()->run_reactor_event_loop();

    return 0;
}

int main(int argc, char *argv[])
{
	try
	{
#if 0
#if 0
		set_new_handler(ace_new_handler);
#else /* ? 0 */
		_set_new_handler(ace_new_handler);
#endif /* 0 */
#endif /* 0 */

		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		fprintf(stderr, "Unhandled error: %s\n", x.what());
	}
	catch(...)
	{
		fprintf(stderr, "Unhandled unknown error\n");
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void cancel_proc(void *param, long code, event_id id, void *arg)
{
	std::cout << "Cancelled: param: " << param
			  << "; code: " << code
			  << "; id: " << reinterpret_cast<unsigned long>(id)
			  << "; arg: " << reinterpret_cast<unsigned long>(arg)
			  << std::endl;
}

/* ////////////////////////////////////////////////////////////////////////// */
