/* /////////////////////////////////////////////////////////////////////////////
 * File:        interface_ptr_test.cpp
 *
 * Purpose:     Implementation file for the interface_ptr_test project.
 *
 * Created:     17th May 2005
 * Updated:     18th May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>
#include <stlsoft/type_traits.hpp>

/* COMSTL Header Files */
#include <comstl.h>
#include <comstl/interface_ptr.hpp>
#include <comstl/interface_cast.hpp>


/* Standard C Header Files */
#include <stdio.h>

/* Standard C++ Header Files */
#include <exception>

#ifdef __WATCOMC__
namespace std
{
	using ::exception;
}
#endif /* __WATCOMC__ */


/* /////////////////////////////////////////////////////////////////////////////
 * Compiler warnings
 */

#if defined(__BORLANDC__)
# pragma warn -8004
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

typedef ::comstl::interface_ptr<IUnknown>	IUnknown_ptr;
typedef ::comstl::interface_ptr<IStream>	IStream_ptr;
typedef ::comstl::interface_ptr<IMalloc>	IMalloc_ptr;


inline void assign_destructive_cast(IStream_ptr *psrc, IStream *pitf)
{
	*psrc = IStream_ptr(pitf, false);
}

inline void assign_destructive_cast(IStream **psrc, IStream *pitf)
{
	*psrc = pitf;
}


template<	typename DEST
		,	typename SRC
		>
class destructive_caster
{
public:
	typedef ss_typename_type_k stlsoft::base_type_traits<DEST>::base_type	pointer_type;

public:
	destructive_caster(SRC *psrc)
		: m_psrc(psrc)
		, m_ptr(NULL)
	{}
	~destructive_caster()
	{
		assign_destructive_cast(m_psrc, m_ptr);	// Here's the destructive bit
	}

	operator DEST() const
	{
		return const_cast<DEST>(&m_ptr);
	}

private:
	SRC				*m_psrc;
	pointer_type	m_ptr;
};

template<	typename DEST
		,	typename SRC
		>
inline destructive_caster<DEST, SRC> destructive_cast(SRC *src)
{
	return destructive_caster<DEST, SRC>(src);
}

/* ////////////////////////////////////////////////////////////////////////// */

namespace comstl2
{
	template <typename I>
	inline ::comstl::interface_ptr<I> interface_cast_impl(I *pi)
	{
		return ::comstl::interface_ptr<I>(pi, true);
	}

	template<	typename IDest
			,	typename ISrc
			>
	inline ::comstl::interface_ptr<IDest> interface_cast(::comstl::interface_ptr<ISrc> src)
	{
		ISrc	*psrc	=	::stlsoft::get_ptr(src);	// For some reason this has to be separate, otherwise VC++ whinges

		::comstl::interface_cast_noaddref<IDest*>	ptr(psrc);	// For some reason, this has to be separate, otherwise G++ has a spit

		return interface_cast_impl(::stlsoft::get_ptr(ptr));
	}

} // namespace comstl2

/* ////////////////////////////////////////////////////////////////////////// */

static int main_(int /* argc */, char ** /*argv*/)
{
#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	puts("interface_ptr_test: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

	/* . */
	{
		LPSTREAM	pstm;
		HRESULT		hr	=	::CreateStreamOnHGlobal(NULL, true, &pstm);

		IStream_ptr	p(pstm, false);

		IStream_ptr	p2(p);
		IStream_ptr	p3;

		p3 = p;
	}

	{
		IStream_ptr	p;
		HRESULT		hr	=	::CreateStreamOnHGlobal(NULL, true, destructive_cast<IStream**>(&p));

		IStream_ptr	p2(p);
		IStream_ptr	p3;

		p3 = p;
	}

	{
		LPSTREAM	pstm;
		HRESULT		hr	=	::CreateStreamOnHGlobal(NULL, true, destructive_cast<IStream**>(&pstm));

		IStream_ptr	p(pstm, false);

		IStream_ptr	p2(p);
		IStream_ptr	p3;

		p3 = p;
	}

	{
		LPSTREAM		pstm;
		HRESULT			hr	=	::CreateStreamOnHGlobal(NULL, true, &pstm);

		IStream_ptr		p(pstm, false);

		IUnknown_ptr	punk	=	comstl2::interface_cast<IUnknown>(p);

		IStream_ptr		pstm2	=	comstl2::interface_cast<IStream>(punk);
	}

	{
		LPSTREAM		pstm;
		HRESULT			hr	=	::CreateStreamOnHGlobal(NULL, true, &pstm);

		IStream_ptr		p(pstm, false);

		IUnknown_ptr	punk	=	comstl2::interface_cast<IUnknown>(p);

		IMalloc_ptr		pstm2	=	comstl2::interface_cast<IMalloc>(punk);
	}


    return 0;
}

int main(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		fprintf(stderr, "Unhandled error: %s\n", x.what());
	}
	catch(...)
	{
		fprintf(stderr, "Unhandled unknown error\n");
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
