/* /////////////////////////////////////////////////////////////////////////////
 * File:        filtered_range_test.cpp
 *
 * Purpose:     Implementation file for the filtered_range_test project.
 *
 * Created:     26th October 2004
 * Updated:     19th March 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* RangeLib Header Files */
#include <rangelib/cstring_range.hpp>
#include <rangelib/algorithms.hpp>
#include <rangelib/filtered_range.hpp>
#include <rangelib/integral_range.hpp>
#include <rangelib/sequence_range.hpp>

/* Standard C++ Header Files */
#include <iostream>

using std::cout;
using std::cerr;
using std::endl;

#include <vector>

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */


/* ////////////////////////////////////////////////////////////////////////// */

struct is_vowel
{
public:
	bool operator ()(char ch) const
	{
		switch(ch)
		{
			default:
				return false;
			case	'a':
			case	'e':
			case	'i':
			case	'o':
			case	'u':
			case	'A':
			case	'E':
			case	'I':
			case	'O':
			case	'U':
				return true;
		}
	}
};

struct is_odd
{
public:
	bool operator ()(int i) const
	{
		return 0 != (i % 2);
	}
};

/* ////////////////////////////////////////////////////////////////////////// */

int main(int /* argc */, char ** /*argv*/)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "filtered_range_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	/* . */
	{
		cout << "vector<int>" << endl;

		typedef std::vector<int>									int_vector_t;
		typedef rangelib::sequence_range<int_vector_t>				int_range_t;
//		typedef	rangelib::filtered_range<int_range_t, is_odd>		filtered_int_range_t;


		int_vector_t				iv;

		rangelib::r_copy(rangelib::make_integral_range(-10, +10, 1), std::back_inserter(iv));

		int_range_t					ir(iv);

		cout << "  Unfiltered: ";
		rangelib::r_copy(ir, std::ostream_iterator<int>(cout, " "));
		cout << endl;

		cout << "  Filtered:   ";
		rangelib::r_copy(rangelib::filter_range(ir, is_odd()), std::ostream_iterator<int>(cout, " "));
		cout << endl;

		cout << endl;
	}

	{
		cout << "c-string" << endl;

		typedef	rangelib::cstring_range<char>						cstring_range_t;
		typedef	rangelib::filtered_range<cstring_range_t, is_vowel>	filtered_cstring_range_t;

		cstring_range_t				r("Lemon sherbert");

		cout << "  Unfiltered: ";
		rangelib::r_copy(r, std::ostream_iterator<char>(cout, " "));
		cout << endl;

		cout << "  Filtered:   ";
		rangelib::r_copy(filtered_cstring_range_t(r), std::ostream_iterator<char>(cout, " "));
		cout << endl;

		cout << endl;
	}

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
