/* /////////////////////////////////////////////////////////////////////////////
 * File:        indexed_range_test.cpp
 *
 * Purpose:     Implementation file for the indexed_range_test project.
 *
 * Created:     29th September 2004
 * Updated:     18th May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* RangeLib Header Files */
#include <rangelib/rangelib.hpp>
#include <rangelib/algorithms.hpp>
#include <rangelib/indexed_range.hpp>
#include <rangelib/integral_range.hpp>
#include <rangelib/sequence_range.hpp>

/* Standard C++ Header Files */
#include <iostream>
#include <vector>

using std::cout;
using std::cerr;
using std::endl;

/* Standard C Header Files */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

int main_(int /* argc */, char ** /*argv*/)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "indexed_range_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	static const int	ints[] = 
	{
		-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9
	};

	{
		typedef ::std::vector<int>						vector_t;

		vector_t								v(&ints[0], &ints[0] + stlsoft_num_elements(ints));

		typedef ::rangelib::sequence_range<vector_t>	sequence_t;

		::rangelib::indexed_range<sequence_t>	ir(::rangelib::make_sequence_range(v));

		for(; ir; ++ir)
		{
			printf("%d: %d\n", ir.index(), *ir);
		}
	}

	{
		typedef rangelib::integral_range<int>	int_range_t;

		{ for(rangelib::indexed_range<int_range_t> r(int_range_t(-20, 20, 4)); r; ++r)
		{
			cout << "Element-#" << r.index() << ": " << *r << endl;
		}}
	}

    return 0;
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
