/* /////////////////////////////////////////////////////////////////////////////
 * File:        sequence_range_test.cpp
 *
 * Purpose:     Implementation file for the sequence_range_test project.
 *
 * Created:     31st October 2003
 * Updated:     30th May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* RangeLib Header Files */
#include <rangelib/rangelib.hpp>
#include <rangelib/algorithms.hpp>
#include <rangelib/sequence_range.hpp>

/* Standard C Header Files */

/* Standard C++ Header Files */
#include <iostream>
#include <vector>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

template <typename I>
struct int_dumper
	: public std::unary_function<I, void>
{
public:
	void operator ()(I i) const
	{
		cout << i << " ";
	}
};

template <typename I>
struct divisible_by_eight
	: public std::unary_function<I, bool>
{
public:
	bool operator ()(I i) const
	{
		return 0 == (i % 8);
	}
};

/* ////////////////////////////////////////////////////////////////////////// */

int main_(int /* argc */, char ** /*argv*/)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "sequence_range_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	{ // vector<int>
		typedef std::vector<int>						container_t;
		typedef rangelib::sequence_range<	container_t
#if !defined(__STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT) || \
    !defined(STLSOFT_CF_HAS_MEMBER_TYPE_SUPPORTED)
										,	rangelib::sequence_range_traits<container_t>
#endif /* !__STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */
										>				range_t;

#if 0
		{
			cout << endl;

			cout << "container_t has iterator member:        " << (stlsoft::member_traits<container_t>::has_member_iterator ? "true" : "false") << endl;
			cout << "container_t has const_iterator member:  " << (stlsoft::member_traits<container_t>::has_member_const_iterator ? "true" : "false") << endl;
# if defined(__STLSOFT_COMPILER_IS_MWERKS)
			cout << "container_t has reference member:       " << (stlsoft::member_traits<container_t>::has_member_reference ? "true" : "false") << endl;
			cout << "container_t has const_reference member: " << (stlsoft::member_traits<container_t>::has_member_const_reference ? "true" : "false") << endl;
# endif /* __STLSOFT_COMPILER_IS_MWERKS */

			cout << "sizeof(container_t::iterator):    " << stlsoft::size_of<container_t::iterator>::value << endl;
			cout << "sizeof(range_t::iterator):        " << stlsoft::size_of<range_t::iterator>::value << endl;
			cout << "sizeof(container_t::reference):   " << stlsoft::size_of<container_t::reference>::value << endl;
			cout << "sizeof(range_t::reference):       " << stlsoft::size_of<range_t::reference>::value << endl;

			cout << "typeof(container_t) " << (stlsoft::is_same_type<container_t, range_t>::value ? "==" : "!=") << " typeof(range_t)" << endl;

			cout << "typeof(container_t::iterator) " << (stlsoft::is_same_type<container_t::iterator, range_t::iterator>::value ? "==" : "!=") << " typeof(range_t::iterator)" << endl;

			cout << endl;
		}
#endif /* 0 */

		container_t	container;

		container.push_back(10);
		container.push_back(-3);
		container.push_back(27);
		container.push_back(32);
		container.push_back(-101);
		container.push_back(19);
		container.push_back(104);

		cout << "Testing sequence_range<vector<int>>" << endl;

		range_t	r(container);
		range_t	r_(r);

#if 1
		cout << "Manual enumeration:" << endl;
		{ for(range_t r2(r); r2; ++r2)
		{
			cout << *r2 << " ";
		}}
		cout << endl;
		cout << endl;

#if 1
		cout << "Enumeration via r_copy():" << endl;
		rangelib::r_copy(r, std::ostream_iterator<int>(cout, " "));
		cout << endl;
		cout << endl;

		cout << "Enumeration via r_for_each():" << endl;
		rangelib::r_for_each(r, int_dumper<int>());
		cout << endl;
		cout << endl;
#endif /* 0 */

		cout << "Sum, via manual enumeration:                 ";
		int	total0 = 0;
		{ for(range_t r2(r); r2; ++r2)
		{
			total0 += *r2;
		}}
		cout << total0 << endl;
		cout << endl;

		cout << "Sum, using r_accumulate():                   " << rangelib::r_accumulate(r, 0) << endl;
		cout << endl;

		cout << "Number of elements, via manual enumeration:  ";
		int	distance0 = 0;
		{ for(range_t r2(r); r2; ++r2, ++distance0)
		{
		}}
		cout << distance0 << endl;
		cout << endl;

		cout << "Number of elements, using r_distance():      " << rangelib::r_distance(r) << endl;
		cout << endl;

		cout << "Number of 4s, via manual enumeration:        ";
		int	count0 = 0;
		{ for(range_t r2(r); r2; ++r2)
		{
			if(4 == *r2)
			{
				++count0;
			}
		}}
		cout << count0 << endl;
		cout << endl;

		cout << "Number of 4s, using r_count():               " << rangelib::r_count(r, 4) << endl;
		cout << endl;

		cout << "Number of %8s, via manual enumeration:       ";
		int	count_if0 = 0;
		{ for(range_t r2(r); r2; ++r2)
		{
			if(0 == (*r2 % 8))
			{
				++count_if0;
			}
		}}
		cout << count_if0 << endl;
		cout << endl;

		cout << "Enumeration of %8s via r_copy_if():" << endl;
		rangelib::r_copy_if(r, std::ostream_iterator<int>(cout, " "), divisible_by_eight<int>());
		cout << endl;
		cout << endl;


		cout << "Number of %8s, using r_count_if():           " << rangelib::r_count_if(r, divisible_by_eight<int>()) << endl;
		cout << endl;

		cout << "Minimum value, using r_min_element():        " << rangelib::r_min_element(r) << endl;
		cout << endl;

		cout << "Maximum value, using r_max_element():        " << rangelib::r_max_element(r) << endl;
		cout << endl;

/*
{
			container_t::iterator	it;
			range_t::iterator		rit;
			range_t					r(container);

			it	=	r.begin();
			rit	=	r.begin();

			*it *= 2;
			*rit *= 2;
		}

		{
			container_t::reference	ref = *container.begin();
			range_t::reference		rref = *container.begin();
			range_t					r(container);

			ref *= 2;
			rref *= 2;
		}
*/

		cout << "Double each entry" << endl;
		{ for(range_t r3(r); r3; ++r3)
		{
			*r3 *= 2;
		}}
		
		cout << "Enumeration via r_copy():" << endl;
		rangelib::r_copy(r, std::ostream_iterator<int>(cout, " "));
		cout << endl;
		cout << endl;
#endif /* 0 */

	}

#if 1
	{ // Test different flavours of const-ness

		typedef std::vector<int>							container_t;
		typedef rangelib::sequence_range<container_t const>	range_t;

		container_t	container_;

		container_.push_back(10);
		container_.push_back(-3);
		container_.push_back(27);
		container_.push_back(19);
		container_.push_back(-101);

		cout << "Testing sequence_range<vector<int>>" << endl;

		container_t const	&container	=	container_;
		/* const  */range_t		r(container);

		cout << "Enumeration via r_copy():" << endl;
		rangelib::r_copy(r, std::ostream_iterator<int>(cout, " "));
		cout << endl;
		cout << endl;


	}
#endif /* 0 */

    return 0;
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
