/* /////////////////////////////////////////////////////////////////////////////
 * File:        indexed_iterator_test.cpp
 *
 * Purpose:     Implementation file for the indexed_iterator_test project.
 *
 * Created:     21st March 2005
 * Updated:     21st March 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>
#include <stlsoft/string_access.hpp>
#include <stlsoft/indexed_iterator.hpp>

/* Standard C Header Files */
#include <stdlib.h>

/* Standard C++ Header Files */
#if defined(__DMC__)
# include <iostream.h>
#else /* ? __DMC__ */
# include <iostream>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;
#endif /* __DMC__ */
#include <list>
#include <string>
#include <vector>

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv);

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */


/* ////////////////////////////////////////////////////////////////////////// */

int main_(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int                 iRet    =   0;
	int					bVerbose	=	true;
    int                 i;

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "indexed_iterator_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
				switch(arg[1])
				{
					case	's':
						bVerbose	=	false;
						break;
					case	'v':
						bVerbose	=	true;
						break;
					default:
						usage(1, "Invalid argument(s) specified", i, argc, argv);
						break;
				}
			}
		}
		else
		{
			/* other arguments */
			usage(1, "Invalid argument(s) specified", i, argc, argv);
		}
	}

	/* . */
	typedef ::std::string					string_t;
	typedef ::std::list<string_t>			string_list_t;
	typedef ::std::vector<string_t>			string_vector_t;

	string_list_t	sl;
	string_vector_t	sv;

	sl.push_back("String 1");
	sl.push_back("String 2");
	sl.push_back("String 3");

	sv.push_back("String 1");
	sv.push_back("String 2");
	sv.push_back("String 3");

#if 0
	typedef ::stlsoft::indexed_iterator<string_list_t::const_iterator>			slii_t;
	typedef ::stlsoft::indexed_iterator<string_vector_t::const_iterator>		svii_t;
#else /* ? 0 */
	typedef ::stlsoft::indexed_iterator<string_list_t::/* const_ */iterator>	slii_t;
	typedef ::stlsoft::indexed_iterator<string_vector_t::/* const_ */iterator>	svii_t;
#endif /* 0 */

	{
		slii_t	b(sl.begin());
		slii_t	e(sl.end());

		for(; b != e; ++b)
		{
			printf("%d: %s\n", b.index(), ::stlsoft::c_str_ptr(*b));
		}
	}

	{
		svii_t	b(sv.begin());
		svii_t	e(sv.end());

		for(; b != e; ++b)
		{
			printf("%d: %s\n", b.index(), ::stlsoft::c_str_ptr(*b));
		}
	}

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv)
{

	cerr << endl;
    cerr << "  indexed_iterator_test" << endl << endl;
	cerr << endl;

	if(NULL != reason)
	{
		cerr << "  Error: " << reason << endl;
		cerr << endl;
	}

#if 0
    cerr << "  indexed_iterator_test" << endl << endl;
    cerr << "  USAGE: indexed_iterator_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>" << endl;
    cerr << "    where:" << endl << endl;
    cerr << "    -w             - searches from the current working directory. The default" << endl;
    cerr << "    -p<root-paths> - searches from the given root path(s), separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       -p\"c:\\windows;x:\\bin\"" << endl;
    cerr << "    -r             - deletes readonly files" << endl;
    cerr << "    -h             - searches from the roots of all drives on the system" << endl;
    cerr << "    -d             - displays the path(s) searched" << endl;
    cerr << "    -u             - do not act recursively" << endl;
    cerr << "    -v             - verbose output. Prints time, attributes, size and path. (default)" << endl;
    cerr << "    -s             - succinct output. Prints path only" << endl;
    cerr << "    <search-spec>  - one or more file search specifications, separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       \"*.exe\"" << endl;
    cerr << "                       \"myfile.ext\"" << endl;
    cerr << "                       \"*.exe;*.dll\"" << endl;
    cerr << "                       \"*.xl?;report.*\"" << endl;
	cerr << endl;
	cerr << "  Contact " << _nameSynesisSoftware << endl;
	cerr << "    at \"" << _wwwSynesisSoftware_SystemTools << "\"," << endl;
	cerr << "    or, via email, at \"" << _emailSynesisSoftware_SystemTools << "\"" << endl;
	cerr << endl;
#endif /* 0 */

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
