/* /////////////////////////////////////////////////////////////////////////////
 * File:        member_selector_iterator_test.cpp
 *
 * Purpose:     Implementation file for the member_selector_iterator_test project.
 *
 * Created:     29th May 2005
 * Updated:     30th May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>
#include <stlsoft/member_selector_iterator.hpp>

/* Standard C++ Header Files */
#include <algorithm>
#include <exception>
#include <iterator>
#include <list>
#include <numeric>
#include <iostream>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;

#ifdef __WATCOMC__
namespace std
{
	using ::exception;
}
#endif /* __WATCOMC__ */

/* Standard C Header Files */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

struct Struct1
{
	int		i;
	long	l;
};

struct Struct2
{
	const int	ci;
	const long	cl;

	Struct2(int i, long l)
		: ci(i)
		, cl(l)
	{}
};

/* ////////////////////////////////////////////////////////////////////////// */

template <typename T>
struct doubler
{
public:
	T operator ()(T const &t) const
	{
		return 2 * t;
	}
};

/* ////////////////////////////////////////////////////////////////////////// */

static int main_(int /* argc */, char ** /*argv*/)
{
	using namespace stlsoft;

#if 0
	::Sleep(100000);
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "member_selector_iterator_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	/* . */
#if 1
	// 1. Non-const (ptr); reading only
	{
		static Struct1	items[] = 
		{
				{	1,		2	}
			,	{	11,		22	}
			,	{	111,	222	}
		};

		std::cout	<< std::endl
					<< "non-const (ptr); reading only"
					<< std::endl;

		std::cout << "Items (Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(&items[0], &Struct1::i)
				,	member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::cout << "Sum (Struct1::i): " << std::accumulate(member_select_iterator(&items[0], &Struct1::i), member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i), 0) << std::endl;
	}
#endif /* 0 */

#if 1
	// 2. Non-const (ptr); read/write
	{
		static Struct1	items[] = 
		{
				{	1,		2	}
			,	{	11,		22	}
			,	{	111,	222	}
		};

		std::cout	<< std::endl
					<< "non-const (ptr); read/write"
					<< std::endl;

		std::cout << "Items (Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(&items[0], &Struct1::i)
				,	member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::transform(	member_select_iterator(&items[0], &Struct1::i)
					,	member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i)
					,	member_select_iterator(&items[0], &Struct1::i)
					,	doubler<int>());

		std::cout << "Items (const Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(&items[0], &Struct1::i)
				,	member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::cout	<< "Sum (Struct1::i): "
					<< std::accumulate(member_select_iterator(&items[0], &Struct1::i), member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i), 0) << std::endl;
	}
#endif /* 0 */

#if 1
	// 3. const (ptr); reading only
	{
		static const Struct1	items[] = 
		{
				{	1,		2	}
			,	{	11,		22	}
			,	{	111,	222	}
		};

		std::cout	<< std::endl
					<< "const (ptr); reading only"
					<< std::endl;

		std::cout << "Items (const Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(&items[0], &Struct1::i)
				,	member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::cout << "Sum (const Struct1::i): " << std::accumulate(member_select_iterator(&items[0], &Struct1::i), member_select_iterator(&items[0] + STLSOFT_NUM_ELEMENTS(items), &Struct1::i), 0) << std::endl;
	}
#endif /* 0 */

#if 1
	// 4. non-const list<>; reading only
	{
		static const Struct1	items_[] = 
		{
				{	1,		2	}
			,	{	11,		22	}
			,	{	111,	222	}
		};

		std::cout	<< std::endl
					<< "non-const (list<> iterator); reading only"
					<< std::endl;

		std::list<Struct1>	items(&items_[0], &items_[0] + STLSOFT_NUM_ELEMENTS(items_));

		std::cout << "Items (const Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(items.begin(), &Struct1::i)
				,	member_select_iterator(items.end(), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::cout << "Sum (const Struct1::i): " << std::accumulate(member_select_iterator(items.begin(), &Struct1::i), member_select_iterator(items.end(), &Struct1::i), 0) << std::endl;
	}
#endif /* 0 */

#if 1
	// 5. const list<>; reading only
	{
		static const Struct1	items_[] = 
		{
				{	1,		2	}
			,	{	11,		22	}
			,	{	111,	222	}
		};

		std::cout	<< std::endl
					<< "const (list<> iterator); reading only"
					<< std::endl;

		const std::list<Struct1>	items(&items_[0], &items_[0] + STLSOFT_NUM_ELEMENTS(items_));

		std::cout << "Items (const Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(items.begin(), &Struct1::i)
				,	member_select_iterator(items.end(), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::cout << "Sum (const Struct1::i): " << std::accumulate(member_select_iterator(items.begin(), &Struct1::i), member_select_iterator(items.end(), &Struct1::i), 0) << std::endl;
	}
#endif /* 0 */

#if 1
# ifdef __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
	// 6. non-const list<>; read/write
	{
		static const Struct1	items_[] = 
		{
				{	1,		2	}
			,	{	11,		22	}
			,	{	111,	222	}
		};

		std::cout	<< std::endl
					<< "non-const (list<> iterator); read/write"
					<< std::endl;

		std::list<Struct1>	items(&items_[0], &items_[0] + STLSOFT_NUM_ELEMENTS(items_));

		std::cout << "Items (const Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(items.begin(), &Struct1::i)
				,	member_select_iterator(items.end(), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

#if 1
		std::transform(	member_select_iterator(items.begin(), &Struct1::i)
					,	member_select_iterator(items.end(), &Struct1::i)
					,	member_select_iterator(items.begin(), &Struct1::i)
					,	doubler<int>());
#endif /* 0 */

		std::cout << "Items (const Struct1::i): " << std::endl;

		std::copy(	member_select_iterator(items.begin(), &Struct1::i)
				,	member_select_iterator(items.end(), &Struct1::i)
				,	std::ostream_iterator<int>(std::cout, " "));

		std::cout << std::endl;

		std::cout << "Sum (const Struct1::i): " << std::accumulate(member_select_iterator(items.begin(), &Struct1::i), member_select_iterator(items.end(), &Struct1::i), 0) << std::endl;
	}
# endif /* __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */
#endif /* 0 */

    return 0;
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
