/* /////////////////////////////////////////////////////////////////////////////
 * File:        operator_int_boolean_test.cpp
 *
 * Purpose:     Implementation file for the operator_int_boolean_test project.
 *
 * Created:     1st November 2003
 * Updated:     30th May 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>

#include <stlsoft.h>
//#include <stlsoft_meta.h>
#include <stlsoft_operator_bool.h>

stlsoft_ns_using(operator_bool_generator)

#include <typeinfo>

//#define	USE_OP_INT
//#define	USE_OP_VOID_STAR
//#define	USE_OP_BOOL
#define	USE_OP_PTR_TO_PRIV_STRUCT
//#define	USE_OP_PTR_TO_PRIV_STRUCT_MEMBER

//#define	PROVIDE_OP_NOT

/* ////////////////////////////////////////////////////////////////////////// */

#if defined(USE_OP_INT)
# define TYPE_STRING			"operator int()"
#elif defined(USE_OP_VOID_STAR)
# define TYPE_STRING			"operator void*()"
#elif defined(USE_OP_BOOL)
# define TYPE_STRING			"operator bool()"
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT)
# define TYPE_STRING			"operator boolean*()"
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT_MEMBER)
# define TYPE_STRING			"operator int boolean::*()"
#else
# error Must defined one of USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL, USE_OP_PTR_TO_PRIV_STRUCT or USE_OP_PTR_TO_PRIV_STRUCT_MEMBER
#endif /* USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL USE_OP_PTR_TO_PRIV_STRUCT_MEMBER */

#ifdef PROVIDE_OP_NOT
# define NOT_STRING				" + operator !()"
#else
# define NOT_STRING				""
#endif /* PROVIDE_OP_NOT */

/* ////////////////////////////////////////////////////////////////////////// */

#if 0

// Borland - can't interpret int boolean::* const for the ! expression

#if defined(__STLSOFT_COMPILER_IS_BORLAND)
# define OP_NOT_REQUIRED
#elif defined(__STLSOFT_COMPILER_IS_COMO)
#elif defined(__STLSOFT_COMPILER_IS_DMC)
#elif defined(__STLSOFT_COMPILER_IS_GCC)
#elif defined(__STLSOFT_COMPILER_IS_INTEL)
# if defined(_MSC_VER) && \
	_MSC_VER < 1300
//#  define TRISTATE_IN_TEMPLATE
# endif /* _MSC_VER < 1300 */
#elif defined(__STLSOFT_COMPILER_IS_MSVC)
# if defined(_MSC_VER) && \
	_MSC_VER < 1300
//#  define TRISTATE_IN_TEMPLATE
# endif /* _MSC_VER < 1300 */
# if _MSC_VER < 1100
#  define OP_NOT_REQUIRED
# endif
#elif defined(__STLSOFT_COMPILER_IS_MWERKS)
#elif defined(__STLSOFT_COMPILER_IS_VECTORC)
#elif defined(__STLSOFT_COMPILER_IS_WATCOM)
#  define OP_NOT_REQUIRED
#else
# error Unrecognised compiler
#endif


#endif /* 0 */

/* ////////////////////////////////////////////////////////////////////////// */

// There are two problems to this:
//
// - Some compilers cannot use the int boolean::*() in a ! expression. In this
//   case an operator ! () is supplied. The compilers are Borland, Watcom and VC++ 4.2-
// - Some compilers have a tri-state behaviour, evaluating both (x) and (!x) to be
//   true, or to be false. In this case, operator bool() is provided. The compilers
//   are VC++ 6.0- (and Intel when _MSC_VER). Intel treats (!x) as true when x is
//   false in template only. VC++ treats (x) and (!x) as true when x is true, and
//   (x) and (!x) as false when x is false, in template or non-template

/* ////////////////////////////////////////////////////////////////////////// */

#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
# define STLSOFT_CF_OPERATOR_NOT_VIA_OPERATOR_POINTER_TO_MEMBER_SUPPORT
#endif /* __STLSOFT_COMPILER_IS_BORLAND */


class regular_class
{
public:
	typedef regular_class		class_type;

	static char const *what() { return typeid(class_type).name(); }

public:
	ss_explicit_k regular_class(bool b)
		: m_b(b)
	{}

#if defined(USE_OP_INT)
public:
	operator int() const
	{
		return m_b;
	}
#elif defined(USE_OP_VOID_STAR)
public:
	operator void *() const
	{
		return m_b ? const_cast<class_type*>(this) : NULL;
	}
#elif defined(USE_OP_BOOL)
public:
	operator bool() const
	{
		return m_b;
	}
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT)
private:
	struct boolean {};
public:
	operator boolean *() const
	{
		boolean	b;

		return m_b ? &b : NULL;
	}
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT_MEMBER)
private:
	struct boolean { int i; };
public:
	operator int boolean::*() const
	{
		return m_b ? &boolean::i : NULL;
	}
#else
# error Must defined one of USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL, USE_OP_PTR_TO_PRIV_STRUCT or USE_OP_PTR_TO_PRIV_STRUCT_MEMBER
#endif /* USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL USE_OP_PTR_TO_PRIV_STRUCT_MEMBER */

# ifdef PROVIDE_OP_NOT
	bool operator !() const
	{
		return !m_b;
	}
# endif /* PROVIDE_OP_NOT */

private:
	bool	m_b;
};


#if defined(USE_OP_INT)
# ifdef PROVIDE_OP_NOT
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
#  define REGULAR_CLASS_2_CW7
#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# else
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
#  define REGULAR_CLASS_2_CW7
#  define REGULAR_CLASS_2_CW8
#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# endif /* PROVIDE_OP_NOT */
#elif defined(USE_OP_VOID_STAR)
# ifdef PROVIDE_OP_NOT
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
//#  define REGULAR_CLASS_2_CW7
//#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
//#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# else
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
//#  define REGULAR_CLASS_2_CW7
//#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
//#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# endif /* PROVIDE_OP_NOT */
#elif defined(USE_OP_BOOL)
# ifdef PROVIDE_OP_NOT
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
#  define REGULAR_CLASS_2_CW7
#  define REGULAR_CLASS_2_CW8
#  define REGULAR_CLASS_2_GCC32
#  define REGULAR_CLASS_2_ICL6
#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# else
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
#  define REGULAR_CLASS_2_CW7
#  define REGULAR_CLASS_2_CW8
#  define REGULAR_CLASS_2_GCC32
#  define REGULAR_CLASS_2_ICL6
#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# endif /* PROVIDE_OP_NOT */
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT)
# ifdef PROVIDE_OP_NOT
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
//#  define REGULAR_CLASS_2_CW7
//#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
//#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# else
//#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
//#  define REGULAR_CLASS_2_CW7
//#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
//#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# endif /* PROVIDE_OP_NOT */
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT_MEMBER)
# ifdef PROVIDE_OP_NOT
#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
//#  define REGULAR_CLASS_2_CW7
//#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
//#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# else
#  define REGULAR_CLASS_2_BCC56
//#  define REGULAR_CLASS_2_BCC564
//#  define REGULAR_CLASS_2_CW7
//#  define REGULAR_CLASS_2_CW8
//#  define REGULAR_CLASS_2_GCC32
//#  define REGULAR_CLASS_2_ICL6
//#  define REGULAR_CLASS_2_ICL7
//#  define REGULAR_CLASS_2_VC6
//#  define REGULAR_CLASS_2_VC7
//#  define REGULAR_CLASS_2_VC71
# endif /* PROVIDE_OP_NOT */
#else
# error Must defined one of USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL, USE_OP_PTR_TO_PRIV_STRUCT or USE_OP_PTR_TO_PRIV_STRUCT_MEMBER
#endif /* USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL USE_OP_PTR_TO_PRIV_STRUCT_MEMBER */



#if defined(__STLSOFT_COMPILER_IS_BORLAND) && \
	__BORLANDC__ == 0x0560
# ifdef REGULAR_CLASS_2_BCC56
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_BCC56 */
#elif defined(__STLSOFT_COMPILER_IS_BORLAND) && \
	__BORLANDC__ == 0x0564
# ifdef REGULAR_CLASS_2_BCC564
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_BCC564 */
#elif defined(__STLSOFT_COMPILER_IS_DMC)
#elif (defined(__STLSOFT_COMPILER_IS_GCC) && \
	__GNUC__ >= 3)
# ifdef REGULAR_CLASS_2_GCC32
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_GCC32 */
#elif (defined(__STLSOFT_COMPILER_IS_INTEL) && \
	__INTEL_COMPILER == 600)
# ifdef REGULAR_CLASS_2_ICL6
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_ICL6 */
#elif (defined(__STLSOFT_COMPILER_IS_INTEL) && \
	__INTEL_COMPILER == 700)
# ifdef REGULAR_CLASS_2_ICL7
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_ICL7 */
#elif (defined(__STLSOFT_COMPILER_IS_INTEL) && \
	__INTEL_COMPILER == 800)
# ifdef REGULAR_CLASS_2_ICL8
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_ICL8 */
#elif (defined(__STLSOFT_COMPILER_IS_MWERKS) && \
	(__MWERKS__ & 0xFF00) == 0x2400)
# ifdef REGULAR_CLASS_2_CW7
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_CW7 */
#elif (defined(__STLSOFT_COMPILER_IS_MWERKS) && \
	(__MWERKS__ & 0xFF00) == 0x3000)
# ifdef REGULAR_CLASS_2_CW8
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_CW8 */
#elif (defined(__STLSOFT_COMPILER_IS_MSVC) && \
	_MSC_VER == 1020)
# ifdef REGULAR_CLASS_2_VC42
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_VC42 */
#elif (defined(__STLSOFT_COMPILER_IS_MSVC) && \
	_MSC_VER == 1100)
# ifdef REGULAR_CLASS_2_VC5
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_VC5 */
#elif (defined(__STLSOFT_COMPILER_IS_MSVC) && \
	_MSC_VER == 1200)
# ifdef REGULAR_CLASS_2_VC6
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_VC6 */
#elif (defined(__STLSOFT_COMPILER_IS_MSVC) && \
	_MSC_VER == 1300)
# ifdef REGULAR_CLASS_2_VC7
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_VC7 */
#elif (defined(__STLSOFT_COMPILER_IS_MSVC) && \
	_MSC_VER == 1310)
# ifdef REGULAR_CLASS_2_VC71
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_VC71 */
#elif defined(__STLSOFT_COMPILER_IS_WATCOM)
# ifdef REGULAR_CLASS_2_WATCOM
#  define REGULAR_CLASS_2
# endif /* REGULAR_CLASS_2_WATCOM */
#else
# error Compiler not recognised
#endif /* compiler */



#ifdef REGULAR_CLASS_2

class regular_class_2
{
public:
	typedef regular_class_2		class_type;

	static char const *what() { return typeid(class_type).name(); }

public:
	ss_explicit_k regular_class_2(bool b)
		: m_b(b)
	{}

	operator char const *() const;

#if defined(USE_OP_INT)
public:
	operator int() const
	{
		return m_b;
	}
#elif defined(USE_OP_VOID_STAR)
public:
	operator void *() const
	{
		return m_b ? const_cast<class_type*>(this) : NULL;
	}
#elif defined(USE_OP_BOOL)
public:
	operator bool() const
	{
		return m_b;
	}
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT)
private:
	struct boolean {};
public:
	operator boolean *() const
	{
		boolean	b;

		return m_b ? &b : NULL;
	}
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT_MEMBER)
private:
	struct boolean { int i; };
public:
	operator int boolean::*() const
	{
		return m_b ? &boolean::i : NULL;
	}
#else
# error Must defined one of USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL, USE_OP_PTR_TO_PRIV_STRUCT or USE_OP_PTR_TO_PRIV_STRUCT_MEMBER
#endif /* USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL USE_OP_PTR_TO_PRIV_STRUCT_MEMBER */

# ifdef PROVIDE_OP_NOT
	bool operator !() const
	{
		return !m_b;
	}
# endif /* PROVIDE_OP_NOT */

private:
	bool	m_b;
};

#endif /* REGULAR_CLASS_2 */



template <class T>
class template_class
{
public:
	typedef template_class<T>	class_type;

	static char const *what() { return typeid(class_type).name(); }

public:
	ss_explicit_k template_class(bool b)
		: m_b(b)
	{}

#if defined(USE_OP_INT)
public:
	operator int() const
	{
		return m_b;
	}
#elif defined(USE_OP_VOID_STAR)
public:
	operator void *() const
	{
		return m_b ? const_cast<class_type*>(this) : NULL;
	}
#elif defined(USE_OP_BOOL)
public:
	operator bool() const
	{
		return m_b;
	}
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT)
private:
	struct boolean {};
public:
	operator boolean *() const
	{
		boolean	b;

		return m_b ? &b : NULL;
	}
#elif defined(USE_OP_PTR_TO_PRIV_STRUCT_MEMBER)
private:
	struct boolean { int i; };
public:
	operator int boolean::*() const
	{
		return m_b ? &boolean::i : NULL;
	}
#else
# error Must defined one of USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL, USE_OP_PTR_TO_PRIV_STRUCT or USE_OP_PTR_TO_PRIV_STRUCT_MEMBER
#endif /* USE_OP_INT, USE_OP_VOID_STAR, USE_OP_BOOL USE_OP_PTR_TO_PRIV_STRUCT_MEMBER */

# ifdef PROVIDE_OP_NOT
	bool operator !() const
	{
		return !m_b;
	}
# endif /* PROVIDE_OP_NOT */

private:
	bool	m_b;
};

/* ////////////////////////////////////////////////////////////////////////// */

#if 0
class PortableExpressibleThing
{
public:
	typedef PortableExpressibleThing	class_type;

	static char const *what() { return typeid(class_type).name(); }

public:
	PortableExpressibleThing(bool b)
		: m_b(b)
	{}

private:
#if 0
	typedef operator_bool_generator<class_type>	boolean_type;
	typedef boolean_type::return_type			operator_bool_type;
#else /* ? 0 */
	typedef stlsoft_ns_qual(operator_bool_generator)<class_type>::class_type	boolean_type;
#endif /* 0 */
public:
	operator boolean_type::return_type() const
	{
		return boolean_type::translate(m_b);
	}

private:
	bool	m_b;
};
#endif /* 0 */

/* ////////////////////////////////////////////////////////////////////////// */

struct ConvertibilityTest
{
	explicit ConvertibilityTest(void const *)
		: type("Implicitly converible to void const*")
	{}
	explicit ConvertibilityTest(int)
		: type("Implicitly converible to int")
	{}
#ifdef __STLSOFT_CF_NATIVE_BOOL_SUPPORT
	explicit ConvertibilityTest(bool)
		: type("Implicitly converible to bool")
	{}
#endif /* __STLSOFT_CF_NATIVE_BOOL_SUPPORT */
	explicit ConvertibilityTest(...)
		: type("Not implicitly converible to void*, bool or int")
	{}

	char const	*type;
};

template <class T>
void TestClass(T t, T f)
{
	printf("Testing: %s\n", T::what());

	int	successes	=	0;
	int	failures	=	0;

	T	t2(t);

#if defined(USE_OP_INT) || \
	defined(USE_OP_BOOL)
	int	i	=	t;

	int	nonsense = t + t2;
#endif /* USE_OP_INT || USE_OP_BOOL */

	t == t2;
	t != t2;

	// (i) Single expression tests

	if(t)
	{
		++successes;
	}
	else
	{
		++failures;
		printf("Failed to identity a true item as true\n");
	}

	if(!t)
	{
		++failures;
		printf("Mistakenly identitied a true item as false\n");
	}
	else
	{
		++successes;
	}


	if(!f)
	{
		++successes;
	}
	else
	{
		++failures;
		printf("Failed to identity a false item as false\n");
	}

	if(f)
	{
		++failures;
		printf("Mistakenly identitied a false item as true\n");
	}
	else
	{
		++successes;
	}

	// (ii) Compound expression tests
#if 1
	if(t && f)
	{
		++failures;
		printf("Mistakenly identitied a false item as true, when in conjunction with a true item\n");
	}
	else
	{
		++successes;
	}
#endif /* 0 */

	if(!t && !f)
	{
		++failures;
		printf("Mistakenly identitied a true item as false, when in conjunction with a false item\n");
	}
	else
	{
		++successes;
	}

	// (ii) Compound expression tests
	int	true_	=	1;
	int false_	=	0;

	if(f && true_)
	{
		++failures;
		printf("Mistakenly identitied a false item as true, when in conjunction with a true item\n");
	}
	else
	{
		++successes;
	}

	if(!t && !false_)
	{
		++failures;
		printf("Mistakenly identitied a true item as false, when in conjunction with a false item\n");
	}
	else
	{
		++successes;
	}

	// (iii) Test convertibility
#if !defined(__STLSOFT_COMPILER_IS_WATCOM)
	ConvertibilityTest	ct(t);
	char const			*convertibility = ct.type;
#else /* ? __STLSOFT_COMPILER_IS_WATCOM */
	char const			*convertibility = "";
#endif /* !__STLSOFT_COMPILER_IS_WATCOM */

	printf("operator bool() test: %s; %s%s", __STLSOFT_COMPILER_VERSION_STRING, TYPE_STRING, NOT_STRING);

	printf("; %s; success: %d; failure: %d\n", convertibility, successes, failures);
	printf("\n");
}


/* ////////////////////////////////////////////////////////////////////////// */

int main(int /* argc */, char ** /*argv*/)
{
//	printf("operator_int_boolean_test: %s, %s%s\n\n", __STLSOFT_COMPILER_LABEL_STRING, TYPE_STRING, NOT_STRING);

	TestClass(regular_class(true), regular_class(false));
#ifdef REGULAR_CLASS_2
	TestClass(regular_class_2(true), regular_class_2(false));
#endif /* REGULAR_CLASS_2 */
	TestClass(template_class<int>(true), template_class<int>(false));
//	TestClass(template_class<regular_class>(true), template_class<regular_class>(false));
//	TestClass(PortableExpressibleThing(true), PortableExpressibleThing(false));

#if 0
	/* . */
	{
		printf("regular_class:\n");

		regular_class		t(true);
		regular_class		f(false);

		if(t)
		{
			if(!t)
			{
				fprintf(stderr, "Failed to distinguish between true and false for %s compiler!\n", __STLSOFT_COMPILER_LABEL_STRING);
				stlsoft_message_assert("Failed to distinguish between true and false for "__STLSOFT_COMPILER_LABEL_STRING" compiler!\n", 0);
			}
		}

		if(	t &&
			!t)
		{
			fprintf(stderr, "true and !true !!\n");
		}

		if(t)
		{
			printf("  true\n");
		}
		if(!t)
		{
			printf("  false - wrong!\n");
		}

		if(f)
		{
			printf("  true - wrong!\n");
		}
		if(!f)
		{
			printf("  false\n");
		}
	}

	/* . */
	{
		printf("template_class<int>:\n");

		template_class<int>		t(true);
		template_class<int>		f(false);
		int						true_count	=	0;
		int						false_count	=	0;

//		int			imp_prom	=	t;

		if(t)
		{
			if(!t)
			{
				fprintf(stderr, "Failed to distinguish between true and false for %s compiler!\n", __STLSOFT_COMPILER_LABEL_STRING);
				stlsoft_message_assert("Failed to distinguish between true and false for "__STLSOFT_COMPILER_LABEL_STRING" compiler!\n", 0);
			}
		}

		if(t)
		{
			++true_count;
			printf("  true\n");
		}
		if(!t)
		{
			++false_count;
			printf("  false - wrong!\n");
		}

		if(f)
		{
			++true_count;
			printf("  true - wrong!\n");
		}
		if(!f)
		{
			++false_count;
			printf("  false\n");
		}

		stlsoft_message_assert("True count invalid", true_count == 1);
		stlsoft_message_assert("False count invalid", false_count == 1);
	}
#endif /* 0 */

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
